@@ -1,5 +1,4 @@
-import requests
-
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.oauth2.client import OAuth2Error
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
@@ -24,7 +23,7 @@ class SlackOAuth2Adapter(OAuth2Adapter):
     def get_data(self, token):
         # Verify the user first
         hed = {"Authorization": "Bearer " + token}
-        resp = requests.get(self.identity_url, headers=hed)
+        resp = get_adapter().get_requests_session().get(self.identity_url, headers=hed)
         resp = resp.json()
 
         if not resp.get("ok"):
